
# q-expansion of newform 3696.2.q.c, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3696
weight = 2
poly_data = [36, 0, 88, 0, 61, 0, 14, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 46, 0, 55, 0, 14, 0, 1], 12], [[12, 0, 9, 0, 1, 0, 0, 0], 2], [[0, 38, 0, 41, 0, 12, 0, 1], 4], [[60, 8, 96, -28, 33, -11, 3, -1], 12], [[60, -8, 96, 28, 33, 11, 3, 1], 12], [[96, -4, 138, -28, 39, -11, 3, -1], 12], [[96, 4, 138, 28, 39, 11, 3, 1], 12]]

hecke_ring_character_values = [[463, [1, 0, 0, 0, 0, 0, 0, 0]], [2773, [1, 0, 0, 0, 0, 0, 0, 0]], [2465, [1, 0, 0, 0, 0, 0, 0, 0]], [1585, [-1, 0, 0, 0, 0, 0, 0, 0]], [673, [-1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 0, 0, 0, 0], [0, 1, 0, 1, 0, 0, 0, 0], [0, -1, 0, 0, 0, 1, 0, 0], [1, 1, 0, 1, 0, 1, 0, -1], [0, 0, 0, 0, 0, 0, -1, -1], [1, 0, -1, 0, -1, -1, 0, 0], [1, 0, 1, 0, 2, 2, -1, -1], [0, 0, -2, 0, -1, -1, 0, 0], [0, 0, 0, 0, 1, -1, -1, 1], [0, -3, 0, -1, -1, 1, 0, 0], [2, 0, 0, 0, -2, -2, 0, 0], [3, 0, 1, 0, 2, 2, -1, -1], [0, -4, 0, -2, 0, 0, 0, 0], [0, 1, 0, -3, 1, -1, -1, 1], [0, 0, 0, 0, 3, 3, -2, -2], [0, 4, 0, 0, 2, -2, 0, 0], [-6, 0, 2, 0, 0, 0, 1, 1], [-2, 0, 0, 0, 1, 1, -3, -3], [0, 0, 2, 0, -1, -1, -2, -2], [1, 0, 1, 0, 1, 1, 0, 0], [0, 6, 0, 2, 0, 0, -1, 1], [-1, 0, -3, 0, -1, -1, 2, 2], [0, 6, 0, 2, -1, 1, 1, -1], [0, 0, 0, 0, 1, -1, 1, -1], [-6, 0, 2, 0, 1, 1, -1, -1], [0, 3, 0, -3, 3, -3, -2, 2], [0, -6, 0, 2, -1, 1, 1, -1], [0, 2, 0, 4, 0, 0, -1, 1], [-6, 0, -2, 0, -3, -3, 3, 3], [0, -6, 0, -2, 0, 0, -5, 5], [-3, 0, -1, 0, 3, 3, -4, -4], [6, 0, 2, 0, 3, 3, -1, -1], [5, 0, 1, 0, -2, -2, 3, 3], [0, -2, 0, 4, -2, 2, 2, -2], [0, 2, 0, -4, 1, -1, -2, 2], [0, 1, 0, -1, -4, 4, 1, -1], [2, 0, 2, 0, -1, -1, -1, -1], [-14, 0, -2, 0, -2, -2, 2, 2], [-2, 0, -2, 0, 1, 1, 1, 1], [-6, 0, -6, 0, 0, 0, 2, 2], [0, -1, 0, -7, 4, -4, -3, 3], [0, 0, 2, 0, 3, 3, 0, 0], [0, 4, 0, -4, 0, 0, -2, 2], [0, -12, 0, 0, 1, -1, -3, 3], [0, 5, 0, -1, 3, -3, 2, -2], [0, -8, 0, 2, 2, -2, 2, -2], [0, -5, 0, 1, 3, -3, 0, 0], [-5, 0, -3, 0, 2, 2, 1, 1], [0, 3, 0, 1, 0, 0, -5, 5], [0, -10, 0, 0, 2, -2, 2, -2], [0, 4, 0, -4, -4, 4, 0, 0], [7, 0, 3, 0, 1, 1, 0, 0], [0, 6, 0, 2, -2, 2, 2, -2], [0, -4, 0, 0, -1, 1, -1, 1], [0, -14, 0, -8, 3, -3, -1, 1], [0, 17, 0, -3, 0, 0, 0, 0], [-2, 0, 6, 0, 3, 3, -2, -2], [0, 6, 0, 8, -2, 2, 3, -3], [0, -8, 0, -2, 1, -1, -3, 3], [1, 0, -3, 0, 0, 0, 3, 3], [-4, 0, 0, 0, -1, -1, 5, 5], [7, 0, -1, 0, 2, 2, -3, -3], [0, -17, 0, -1, 1, -1, 1, -1], [0, -4, 0, 4, 0, 0, 8, -8], [-4, 0, 0, 0, 1, 1, 0, 0], [-8, 0, 2, 0, 4, 4, -4, -4], [0, -4, 0, -4, 4, -4, -4, 4], [0, 12, 0, 4, 4, -4, -4, 4], [-4, 0, -4, 0, -2, -2, -3, -3], [0, -12, 0, 0, -1, 1, 7, -7], [0, 14, 0, 0, 1, -1, 3, -3], [0, 17, 0, 7, 1, -1, -4, 4], [0, 10, 0, -6, -3, 3, 0, 0], [-2, 0, -2, 0, -3, -3, 5, 5], [0, -1, 0, -1, -3, 3, -3, 3], [-8, 0, 0, 0, -1, -1, 0, 0], [0, 21, 0, -1, 0, 0, -1, 1], [-14, 0, -2, 0, 1, 1, 1, 1], [-9, 0, 3, 0, -1, -1, -2, -2], [0, 6, 0, 2, 2, -2, -2, 2], [6, 0, -4, 0, 2, 2, -4, -4], [0, -6, 0, -4, -1, 1, 3, -3], [0, 0, 0, 8, 0, 0, 4, -4], [8, 0, 8, 0, -1, -1, -4, -4], [-14, 0, 6, 0, 2, 2, 0, 0], [-6, 0, -2, 0, -5, -5, 3, 3], [0, -4, 0, 4, -4, 4, 6, -6], [-22, 0, -6, 0, 3, 3, -1, -1], [-4, 0, -4, 0, -6, -6, -4, -4], [0, 22, 0, 2, -4, 4, 6, -6], [-14, 0, -2, 0, 2, 2, 2, 2], [0, 0, -4, 0, 0, 0, -4, -4], [0, -2, 0, 2, 3, -3, 1, -1], [0, 0, -6, 0, -4, -4, 4, 4], [-6, 0, -10, 0, 2, 2, -2, -2], [0, -17, 0, -5, 0, 0, -2, 2], [0, 4, 0, 8, -3, 3, 3, -3], [11, 0, -1, 0, -6, -6, 3, 3], [0, -14, 0, 2, 1, -1, -2, 2], [0, 8, 0, -2, 2, -2, -8, 8], [0, 0, 0, -4, -3, 3, -5, 5], [3, 0, -3, 0, -4, -4, -1, -1], [0, 8, 0, -2, 7, -7, 3, -3], [0, 12, 0, -6, 0, 0, 2, -2], [0, 12, 0, 4, -1, 1, -1, 1], [0, -16, 0, -4, 2, -2, 0, 0], [9, 0, 11, 0, -1, -1, -4, -4], [-20, 0, -2, 0, 1, 1, 0, 0], [-1, 0, 3, 0, -3, -3, 6, 6], [-22, 0, 2, 0, -3, -3, -2, -2], [0, -14, 0, 2, 3, -3, 0, 0], [-10, 0, -4, 0, -2, -2, 6, 6], [0, -18, 0, 6, 1, -1, 1, -1], [8, 0, -8, 0, -4, -4, 6, 6], [-2, 0, 0, 0, 2, 2, 0, 0], [0, 0, 0, -4, 7, -7, -7, 7], [0, 1, 0, -7, -3, 3, 3, -3], [-8, 0, 0, 0, -1, -1, 4, 4], [0, -2, 0, -2, -5, 5, 5, -5], [0, -7, 0, 3, 0, 0, -3, 3], [0, 24, 0, 0, 8, -8, 2, -2], [-8, 0, -4, 0, 7, 7, -9, -9], [2, 0, 2, 0, 0, 0, 2, 2], [0, -8, 0, 4, -1, 1, 5, -5], [0, 2, 0, -4, 2, -2, 0, 0], [-10, 0, 0, 0, -6, -6, 2, 2], [0, 1, 0, 1, -9, 9, 9, -9], [0, 3, 0, 1, 3, -3, 0, 0], [14, 0, 6, 0, 10, 10, -1, -1], [0, 4, 0, 6, -6, 6, 6, -6], [0, 4, 0, -4, 2, -2, -4, 4], [12, 0, -4, 0, -4, -4, -2, -2], [14, 0, 4, 0, 8, 8, -6, -6], [21, 0, 7, 0, 5, 5, -6, -6], [-23, 0, 1, 0, -3, -3, -2, -2], [0, 19, 0, -1, 4, -4, 2, -2], [-5, 0, 11, 0, 0, 0, 1, 1], [0, 3, 0, -1, -6, 6, -4, 4], [0, -18, 0, 0, 0, 0, -4, 4], [-7, 0, -3, 0, -6, -6, 5, 5], [0, -22, 0, 0, 0, 0, -4, 4], [-12, 0, 0, 0, -8, -8, 4, 4], [0, 16, 0, 4, 2, -2, 10, -10], [0, 3, 0, 5, 6, -6, 5, -5], [0, -21, 0, 3, 5, -5, -3, 3], [-32, 0, 0, 0, -2, -2, 5, 5], [15, 0, 1, 0, 0, 0, -3, -3], [0, -8, 0, 4, -4, 4, 12, -12], [4, 0, 10, 0, 7, 7, -2, -2], [0, 22, 0, 8, -10, 10, 9, -9], [0, -32, 0, -4, 3, -3, -1, 1], [-16, 0, -4, 0, 0, 0, -4, -4], [6, 0, -6, 0, -6, -6, 4, 4], [-18, 0, -8, 0, 1, 1, 1, 1], [-24, 0, -2, 0, 1, 1, 2, 2], [0, 22, 0, -4, -5, 5, 4, -4], [0, -6, 0, 6, -5, 5, 7, -7], [25, 0, 1, 0, 7, 7, -2, -2], [12, 0, -8, 0, -3, -3, 9, 9], [-14, 0, -4, 0, -11, -11, 7, 7], [0, 18, 0, 8, 0, 0, 0, 0], [0, 2, 0, 4, 1, -1, 10, -10], [0, 16, 0, 12, -4, 4, -2, 2], [-18, 0, 0, 0, 2, 2, -8, -8], [0, 31, 0, 11, -13, 13, 15, -15], [4, 0, 8, 0, -2, -2, 0, 0], [18, 0, 2, 0, 0, 0, 3, 3], [0, -8, 0, -4, 0, 0, -2, 2], [0, 41, 0, 5, -2, 2, 6, -6], [0, -4, 0, -12, 2, -2, -10, 10], [0, -19, 0, -9, 4, -4, -13, 13], [0, 12, 0, -4, 2, -2, 0, 0], [0, -8, 0, -12, 2, -2, 2, -2], [0, -17, 0, -7, 1, -1, 2, -2], [0, 12, 0, 0, 11, -11, 3, -3], [0, 16, 0, 2, 0, 0, 6, -6], [28, 0, 0, 0, 1, 1, 2, 2], [-8, 0, -8, 0, -1, -1, 4, 4], [0, 0, 0, 0, 2, 2, -9, -9], [4, 0, 4, 0, 12, 12, -10, -10], [-37, 0, 1, 0, 3, 3, -2, -2], [-22, 0, 0, 0, 4, 4, -2, -2], [-1, 0, -3, 0, 8, 8, 1, 1], [-12, 0, -10, 0, -5, -5, -2, -2], [0, 3, 0, 3, -8, 8, 8, -8], [0, 30, 0, 6, 1, -1, -2, 2], [0, 2, 0, -6, -7, 7, 5, -5], [0, 36, 0, -8, 0, 0, -2, 2], [4, 0, 4, 0, 4, 4, 2, 2], [0, 16, 0, 16, -7, 7, 13, -13], [0, 16, 0, 8, 0, 0, 0, 0], [20, 0, 14, 0, 10, 10, -6, -6], [0, -7, 0, 1, -8, 8, 12, -12], [0, -14, 0, 6, -3, 3, -1, 1], [0, -22, 0, -2, 3, -3, 1, -1], [0, -4, 0, -8, -2, 2, 2, -2], [10, 0, 4, 0, 4, 4, 2, 2], [-13, 0, -15, 0, 0, 0, 1, 1], [6, 0, 2, 0, 14, 14, 0, 0], [0, 10, 0, 0, 12, -12, -4, 4], [0, 0, -8, 0, -9, -9, 4, 4], [0, -23, 0, -5, -4, 4, -5, 5], [41, 0, 5, 0, -3, -3, 0, 0], [0, 6, 0, 10, -4, 4, 12, -12], [0, -3, 0, -11, 6, -6, -12, 12], [0, 1, 0, -17, 3, -3, -8, 8], [0, 0, 0, -4, 0, 0, -16, 16], [0, -24, 0, -2, -3, 3, 9, -9], [0, 12, 0, -8, -1, 1, 1, -1], [0, 0, 0, 8, 2, -2, 4, -4], [0, -1, 0, -1, 12, -12, -10, 10], [28, 0, 4, 0, 8, 8, -2, -2], [0, -6, 0, 6, -2, 2, 12, -12], [-28, 0, -4, 0, -6, -6, -6, -6], [0, 28, 0, -4, -5, 5, 3, -3], [0, 6, 0, -10, -8, 8, -3, 3], [-7, 0, -1, 0, -3, -3, 6, 6], [4, 0, 6, 0, 3, 3, 6, 6], [8, 0, 4, 0, -3, -3, -2, -2], [0, -18, 0, 6, -5, 5, 6, -6], [-38, 0, -6, 0, -1, -1, 6, 6], [18, 0, -2, 0, -11, -11, 1, 1], [-16, 0, -8, 0, 4, 4, -8, -8], [-45, 0, 5, 0, 4, 4, -1, -1], [0, 18, 0, -12, 4, -4, -3, 3], [0, 28, 0, 16, -1, 1, -1, 1], [-4, 0, 14, 0, 13, 13, -6, -6], [20, 0, 12, 0, 3, 3, 4, 4], [0, -6, 0, -2, -11, 11, 7, -7], [14, 0, 8, 0, -2, -2, 6, 6], [19, 0, -9, 0, -6, -6, 7, 7], [0, 2, 0, 0, -7, 7, 10, -10], [0, 20, 0, 10, 1, -1, -3, 3], [0, 12, 0, -16, -4, 4, -4, 4], [34, 0, 14, 0, 18, 18, -12, -12], [0, 14, 0, 10, 2, -2, 4, -4], [0, 2, 0, 8, 0, 0, -2, 2], [18, 0, -12, 0, -1, -1, -9, -9], [0, -17, 0, 11, -3, 3, 7, -7], [10, 0, 2, 0, 12, 12, 0, 0], [-7, 0, -3, 0, 2, 2, -11, -11], [0, 11, 0, 5, -9, 9, 6, -6], [10, 0, -8, 0, 4, 4, 0, 0], [21, 0, -5, 0, -9, -9, -4, -4], [4, 0, 4, 0, 0, 0, -8, -8], [0, 7, 0, -7, 3, -3, -8, 8], [0, -16, 0, -4, -6, 6, -2, 2], [0, -44, 0, 6, -2, 2, 8, -8], [0, 10, 0, 16, -5, 5, 9, -9], [0, -26, 0, -10, 9, -9, -10, 10], [15, 0, 9, 0, 14, 14, -5, -5], [8, 0, 6, 0, -11, -11, 0, 0], [0, -38, 0, -10, 3, -3, -3, 3], [-10, 0, -2, 0, 1, 1, -1, -1], [0, -16, 0, -16, 12, -12, -12, 12], [-26, 0, 0, 0, -12, -12, 12, 12], [1, 0, -3, 0, 6, 6, -3, -3], [0, -17, 0, -5, 8, -8, -14, 14], [-3, 0, 9, 0, 11, 11, 0, 0], [0, -18, 0, 12, -9, 9, 2, -2], [0, 4, 0, 12, 2, -2, 2, -2], [0, 0, 16, 0, 4, 4, 1, 1], [-38, 0, 2, 0, -4, -4, 3, 3], [0, 10, 0, 6, -7, 7, -3, 3], [0, 18, 0, -14, -5, 5, -9, 9], [36, 0, 0, 0, 3, 3, -2, -2], [0, 2, 0, -18, 7, -7, -3, 3], [0, 12, 0, -6, -8, 8, -2, 2], [0, -8, 0, -4, 15, -15, -11, 11], [0, 19, 0, 5, -6, 6, 11, -11], [4, 0, 8, 0, 12, 12, -4, -4], [0, 0, 0, 0, -9, 9, -5, 5], [0, 18, 0, 2, 16, -16, -5, 5], [-41, 0, 3, 0, 9, 9, -2, -2], [0, -51, 0, -1, 1, -1, 0, 0], [-22, 0, -12, 0, -11, -11, 3, 3], [0, -14, 0, 10, -15, 15, 4, -4], [0, 4, 0, 4, 8, -8, -8, 8], [21, 0, -5, 0, -4, -4, 5, 5], [18, 0, 14, 0, 8, 8, -12, -12], [-8, 0, -16, 0, -10, -10, 2, 2], [-2, 0, 2, 0, -10, -10, 12, 12], [6, 0, -18, 0, 0, 0, -1, -1], [17, 0, 1, 0, 2, 2, -5, -5], [-20, 0, -2, 0, 13, 13, -6, -6], [-38, 0, 2, 0, 10, 10, 0, 0], [0, -6, 0, 4, -10, 10, 0, 0], [0, 9, 0, 7, 5, -5, 0, 0], [19, 0, 13, 0, 15, 15, -2, -2], [-4, 0, -8, 0, -3, -3, 10, 10], [0, 16, 0, -12, 8, -8, -10, 10], [0, -32, 0, -6, -9, 9, -1, 1], [29, 0, 11, 0, -4, -4, 1, 1], [0, -22, 0, -8, 2, -2, -13, 13], [-28, 0, 8, 0, 1, 1, -7, -7], [0, 33, 0, 7, 5, -5, 0, 0], [0, -7, 0, 13, -12, 12, -4, 4], [9, 0, -9, 0, -13, -13, -4, -4], [11, 0, -1, 0, 10, 10, 7, 7], [3, 0, 3, 0, 7, 7, -6, -6], [0, 26, 0, 0, 4, -4, -6, 6], [0, 26, 0, -6, 0, 0, -7, 7]]
